@@ -5,7 +5,8 @@
 #include "primality.h"
 #include "prime_iterator.h"
 #include "utility.h"
-#include "small_factor.h"
+#include "pbrent63.h"
+#include "squfof126.h"
 #include "ecm.h"
 #include "simpqs.h"
 
@@ -151,7 +152,8 @@ int factor(mpz_t input_n, mpz_t* pfactors[], int* pexponents[])
     while ( mpz_cmp_ui(n, tlim*tlim) > 0 && !_GMP_is_prob_prime(n) ) {
       int success = 0;
       int o = get_verbose_level();
-      UV nbits, B1 = 5000;
+      UV B1 = 5000;
+      UV nbits = mpz_sizeinbase(n, 2);
 
       /*
        * This set of operations is meant to provide good performance for
@@ -166,26 +168,33 @@ int factor(mpz_t input_n, mpz_t* pfactors[], int* pexponents[])
        * of many 12-digit or 14-digit primes should take under 10 seconds.
        */
 
-      if (mpz_cmp_ui(n, (unsigned long)(UV_MAX>>4)) < 0) {
-        UV ui_n = mpz_get_ui(n);
-        UV ui_factors[2];
-        if (!mpz_cmp_ui(n, ui_n)) {
-          success = racing_squfof_factor(ui_n, ui_factors, 200000)-1;
-          if (success) {
-            mpz_set_ui(f, ui_factors[0]);
-          } else {
-            if (o > 2) {gmp_printf("UV SQUFOF failed %Zd\n", n);}
-          }
-        }
-        if (success&&o) {gmp_printf("UV SQUFOF found factor %Zd\n", f);o=0;}
+      /* Do a quick path for small inputs */
+
+      if (nbits <= 63) {
+        if (!success) success = pbrent63(n, f, 400000);
+        if (success&&o) {gmp_printf("UV Rho-Brent found factor %Zd\n", f);o=0;}
+      }
+      if (nbits <= 53) {
+        if (!success)  success = squfof126(n, f, 400000);
+        if (success&&o) {gmp_printf("UV SQUFOF126 found factor %Zd\n", f);o=0;}
+      } else if (nbits <= 77) {
+        int sb1 = (nbits < 58) ?  1
+                : (nbits < 63) ?  2
+                : (nbits < 72) ?  4
+                               : 10;
+        if (!success)  success = _GMP_pminus1_factor(n, f, sb1*1000, sb1*10000);
+        if (success&&o) {gmp_printf("p-1 (%dk) found factor %Zd\n",sb1,f);o=0;}
+
+        if (!success)  success = squfof126(n, f, 1000000);
+        if (success&&o) {gmp_printf("SQUFOF126 found factor %Zd\n", f);o=0;}
       }
 
       /* Make sure it isn't a perfect power */
       if (!success)  success = (int)power_factor(n, f);
       if (success&&o) {gmp_printf("perfect power found factor %Zd\n", f);o=0;}
 
-      if (!success)  success = _GMP_pminus1_factor(n, f, 15000, 150000);
-      if (success&&o) {gmp_printf("p-1 (15k) found factor %Zd\n", f);o=0;}
+      if (!success)  success = _GMP_pminus1_factor(n, f, 20000, 200000);
+      if (success&&o) {gmp_printf("p-1 (20k) found factor %Zd\n", f);o=0;}
 
       /* Small ECM to find small factors */
       if (!success)  success = _GMP_ECM_FACTOR(n, f, 200, 4);
@@ -197,7 +206,6 @@ int factor(mpz_t input_n, mpz_t* pfactors[], int* pexponents[])
 
       /* Small p-1 */
       if (!success) {
-        nbits = mpz_sizeinbase(n, 2);
         if (nbits < 100 || nbits >= 160) {
           success = _GMP_pminus1_factor(n, f, 200000, 3000000);
           if (success&&o) {gmp_printf("p-1 (200k) found factor %Zd\n", f);o=0;}
@@ -276,9 +284,6 @@ int factor(mpz_t input_n, mpz_t* pfactors[], int* pexponents[])
       /*
       if (!success)  success = _GMP_pbrent_factor(n, f, 2, 512*1024*1024);
       if (success&&o) {gmp_printf("pbrent (2,512M) found factor %Zd\n", f);o=0;}
-
-      if (!success)  success = _GMP_squfof_factor(n, f, 256*1024*1024);
-      if (success&&o) {gmp_printf("squfof found factor %Zd\n", f);o=0;}
       */
 
       /* Our method of last resort: ECM with high bmax and many curves*/
@@ -1438,201 +1443,6 @@ int _GMP_holf_factor(mpz_t n, mpz_t f, UV rounds)
   return 0;
 }
 
-
-/*----------------------------------------------------------------------
- * GMP version of Ben Buhrow's public domain 9/24/09 implementation.
- * It uses ideas and code from Jason Papadopoulos, Scott Contini, and
- * Tom St. Denis.  Also see the papers of Stephen McMath, Daniel Shanks,
- * and Jason Gower.  Gower and Wagstaff is particularly useful:
- *    http://homes.cerias.purdue.edu/~ssw/squfof.pdf
- *--------------------------------------------------------------------*/
-
-static int shanks_mult(mpz_t n, mpz_t f)
-{
-   /*
-    * use shanks SQUFOF to factor N.
-    *
-    * return 0 if no factor found, 1 if found with factor in f1.
-    *
-    * Input should have gone through trial division to 5.
-    */
-
-   int result = 0;
-   unsigned long j=0;
-   mpz_t b0, bn, imax, tmp, Q0, Qn, P, i, t1, t2, S, Ro, So, bbn;
-
-   if (mpz_cmp_ui(n, 3) <= 0)
-     return 0;
-
-   if (mpz_perfect_square_p(n)) {
-     mpz_sqrt(f, n);
-     return 1;
-   }
-
-   mpz_init(b0);
-   mpz_init(bn);
-   mpz_init(imax);
-   mpz_init(tmp);
-   mpz_init(Q0);
-   mpz_init(Qn);
-   mpz_init(P);
-   mpz_init(i);
-   mpz_init(t1);
-   mpz_init(t2);
-   mpz_init(S);
-   mpz_init(Ro);
-   mpz_init(So);
-   mpz_init(bbn);
-
-   mpz_sqrt(b0, n);
-   mpz_sqrt(tmp, b0);
-   mpz_mul_ui(imax, tmp, 3);
-
-   /* set up recurrence */
-   mpz_set_ui(Q0, 1);
-   mpz_set(P, b0);
-   mpz_mul(tmp, b0, b0);
-   mpz_sub(Qn, n, tmp);
-
-   mpz_add(tmp, b0, P);
-   mpz_tdiv_q(bn, tmp, Qn);
-
-   mpz_set_ui(i, 0);
-   while (1) {
-      j=0;
-      while (1) {
-         mpz_set(t1, P);   /* hold Pn for this iteration */
-         mpz_mul(tmp, bn, Qn);
-         mpz_sub(P, tmp, P);
-         mpz_set(t2, Qn);  /* hold Qn for this iteration */
-         mpz_sub(tmp, t1, P);
-         mpz_mul(tmp, tmp, bn);
-         mpz_add(Qn, Q0, tmp);
-         mpz_set(Q0, t2);  /* remember last Q */
-         mpz_add(tmp, b0, P);
-         mpz_tdiv_q(bn, tmp, Qn);
-
-         if (mpz_even_p(i)) {
-           if (mpz_perfect_square_p(Qn)) {
-             mpz_add_ui(i, i, 1);
-             break;
-           }
-         }
-         mpz_add_ui(i, i, 1);
-
-         if (mpz_cmp(i, imax) >= 0) {
-           result = 0;
-           goto end;
-         }
-      }
-
-      /* reduce to G0 */
-      mpz_sqrt(S, Qn);
-      mpz_sub(tmp, b0, P);
-      mpz_tdiv_q(tmp, tmp, S);
-      mpz_mul(tmp, S, tmp);
-      mpz_add(Ro, P, tmp);
-      mpz_mul(tmp, Ro, Ro);
-      mpz_sub(tmp, n, tmp);
-      mpz_tdiv_q(So, tmp, S);
-      mpz_add(tmp, b0, Ro);
-      mpz_tdiv_q(bbn, tmp, So);
-
-      /* search for symmetry point */
-      while (1) {
-         mpz_set(t1, Ro);  /* hold Ro for this iteration */
-         mpz_mul(tmp, bbn, So);
-         mpz_sub(Ro, tmp, Ro);
-         mpz_set(t2, So);  /* hold So for this iteration */
-         mpz_sub(tmp, t1, Ro);
-         mpz_mul(tmp, bbn, tmp);
-         mpz_add(So, S, tmp);
-         mpz_set(S, t2);   /* remember last S */
-         mpz_add(tmp, b0, Ro);
-         mpz_tdiv_q(bbn, tmp, So);
-
-         /* check for symmetry point */
-         if (mpz_cmp(Ro, t1) == 0)
-            break;
-
-         /* this gets stuck very rarely, but it does happen. */
-         if (++j > 1000000000)
-         {
-            result = -1;
-            goto end;
-         }
-      }
-
-      mpz_gcd(t1, Ro, n);
-      if (mpz_cmp_ui(t1, 1) > 0) {
-         mpz_set(f, t1);
-         /* gmp_printf("GMP SQUFOF found factor after %Zd/%lu rounds: %Zd\n", i, j, f); */
-         result = 1;
-         goto end;
-      }
-   }
-
-   end:
-   mpz_clear(b0);
-   mpz_clear(bn);
-   mpz_clear(imax);
-   mpz_clear(tmp);
-   mpz_clear(Q0);
-   mpz_clear(Qn);
-   mpz_clear(P);
-   mpz_clear(i);
-   mpz_clear(t1);
-   mpz_clear(t2);
-   mpz_clear(S);
-   mpz_clear(Ro);
-   mpz_clear(So);
-   mpz_clear(bbn);
-   return result;
-}
-
-int _GMP_squfof_factor(mpz_t n, mpz_t f, UV rounds)
-{
-   const UV multipliers[] = {
-      3*5*7*11, 3*5*7,  3*5*7*11*13, 3*5*7*13, 3*5*7*11*17, 3*5*11,
-      3*5*7*17, 3*5,    3*5*7*11*19, 3*5*11*13,3*5*7*19,    3*5*7*13*17,
-      3*5*13,   3*7*11, 3*7,         5*7*11,   3*7*13,      5*7,
-      3*5*17,   5*7*13, 3*5*19,      3*11,     3*7*17,      3,
-      3*11*13,  5*11,   3*7*19,      3*13,     5,           5*11*13,
-      5*7*19,   5*13,   7*11,        7,        3*17,        7*13,
-      11,       1 };
-   const size_t sz_mul = sizeof(multipliers)/sizeof(multipliers[0]);
-   size_t i;
-   int result;
-   mpz_t t;
-
-   TEST_FOR_2357(n, f);
-   mpz_init(t);
-   mpz_set_ui(f, 1);
-
-   for (i = 0; i < sz_mul; i++) {
-      UV mult = multipliers[i];
-      /* Only run when 64*m^3 < n */
-      mpz_set_ui(t, mult);
-      mpz_pow_ui(t, t, 3);
-      mpz_mul_ui(t, t, 64);
-      if (mpz_cmp(t, n) >= 0)
-        continue;
-      /* Run with this multiplier */
-      mpz_mul_ui(t, n, mult);
-      result = shanks_mult(t, f);
-      if (result == -1)
-        continue;
-      if ( (result == 1) && (mpz_cmp_ui(f, mult) != 0) ) {
-        unsigned long gcdf = mpz_gcd_ui(NULL, f, mult);
-        mpz_divexact_ui(f, f, gcdf);
-        if (mpz_cmp_ui(f, 1) > 0)
-          break;
-      }
-   }
-   mpz_clear(t);
-   return (mpz_cmp_ui(f, 1) > 0);
-}
-
 /* See if n is a perfect power */
 UV power_factor(mpz_t n, mpz_t f)
 {
